@@ -15,16 +15,13 @@ use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
-use DateTime::HiRes ();
 use DateTime::TimeZone ();
-use List::Util 1.33 qw(all any none);
+use List::MoreUtils qw(all any none);
 use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-use DateTime::Format::Natural::Utils qw(trim);
-
-our $VERSION = '1.24_01';
+our $VERSION = '1.01';
 
 validation_options(
     on_fail => sub
@@ -56,7 +53,6 @@ sub _init
     my %presets = (
         lang          => 'en',
         format        => 'd/m/y',
-        demand_future =>  false,
         prefer_future =>  false,
         time_zone     => 'floating',
     );
@@ -71,12 +67,10 @@ sub _init
     $self->{Daytime} = $opts{daytime} || {};
 
     my $mod = join '::', (__PACKAGE__, 'Lang', uc $self->{Lang});
-    eval "require $mod" or die $@;
+    eval "require $mod"; die $@ if $@;
 
     $self->{data} = $mod->__new();
     $self->{grammar_class} = $mod;
-
-    $self->{mode} = '';
 }
 
 sub _init_check
@@ -84,18 +78,6 @@ sub _init_check
     my $self = shift;
 
     validate(@_, {
-        demand_future => {
-            # SCALARREF due to boolean.pm's implementation
-            type => BOOLEAN | SCALARREF,
-            optional => true,
-            callbacks => {
-                'mutually exclusive' => sub
-                {
-                    return true unless exists $_[1]->{prefer_future};
-                    die "prefer_future provided\n";
-                },
-            },
-        },
         lang => {
             type => SCALAR,
             optional => true,
@@ -104,23 +86,12 @@ sub _init_check
         format => {
             type => SCALAR,
             optional => true,
-            regex => qr!^(?:
-                           (?: (?: [dmy]{1,4}[-./] ){2}[dmy]{1,4} )
-                             |
-                           (?: [dm]{1,2}/[dm]{1,2} )
-                         )$!ix,
+            regex => qr!^(?:[dmy]{1,4}[-./]){2}[dmy]{1,4}$!i,
         },
         prefer_future => {
             # SCALARREF due to boolean.pm's implementation
             type => BOOLEAN | SCALARREF,
             optional => true,
-            callbacks => {
-                'mutually exclusive' => sub
-                {
-                    return true unless exists $_[1]->{demand_future};
-                    die "demand_future provided\n";
-                },
-            },
         },
         time_zone => {
             type => SCALAR | OBJECT,
@@ -142,28 +113,6 @@ sub _init_check
         daytime => {
             type => HASHREF,
             optional => true,
-            callbacks => {
-                'valid daytime' => sub
-                {
-                    my $href = shift;
-                    my %daytimes = map { $_ => true } qw(morning afternoon evening);
-                    if (any { !$daytimes{$_} } keys %$href) {
-                        die "spelling of daytime\n";
-                    }
-                    elsif (any { !defined $href->{$_} } keys %$href) {
-                        die "undefined hour\n";
-                    }
-                    elsif (any { $href->{$_} !~ /^\d{1,2}$/ } keys %$href) {
-                        die "not a valid number\n";
-                    }
-                    elsif (any { $href->{$_} < 0 || $href->{$_} > 23 } keys %$href) {
-                        die "hour out of range\n";
-                    }
-                    else {
-                        return true;
-                    }
-                }
-            },
         },
         datetime => {
             type => OBJECT,
@@ -194,8 +143,6 @@ sub parse_datetime
 
     $self->{input_string} = $self->{date_string};
 
-    $self->{mode} = 'parse';
-
     my $date_string = $self->{date_string};
 
     $self->_rewrite(\$date_string);
@@ -214,52 +161,10 @@ sub parse_datetime
         my $dt = $self->_parse_formatted_md($date_string);
         return $dt if blessed($dt);
 
-        if ($self->{Prefer_future} || $self->{Demand_future}) {
+        if ($self->{Prefer_future}) {
             $self->_advance_future('md');
         }
     }
-    elsif ($date_string =~ /^(\d{4}(?:-\d{2}){0,2})T(\d{2}(?::\d{2}){0,2})(?:[.,](\d+))?(Z|[+-]\d{2}(?::?\d{2})?)?$/) {
-        my ($date, $time, $fractional, $tz) = ($1, $2, $3, $4);
-        my %args;
-
-        if (defined $tz) {
-            if ($tz eq 'Z') {
-                $self->{datetime}->set_time_zone('UTC');
-            } elsif ($tz =~ /^([+-])(\d{2})$/) {
-                $tz = "$1$2:00";
-            } else {
-                $tz =~ s/^([+-])(\d{2}):?(\d{2})$/$1$2:$3/;
-            }
-            $self->{datetime}->set_time_zone($tz);
-        }
-
-        @args{qw(year month day)} = split /-/, $date;
-        $args{$_} ||= 01 foreach qw(month day);
-
-        @args{qw(hour minute second)} = split /:/, $time;
-        $args{$_} ||= 00 foreach qw(minute second);
-
-        if (defined $fractional) {
-            my $nanosecond = $fractional;
-            if (length($nanosecond) < 9) {
-                $nanosecond .= '0' x (9 - length($nanosecond));
-            }
-            $args{nanosecond} = int($nanosecond);
-        }
-
-        my $valid_date = $self->_check_date(map $args{$_}, qw(year month day));
-        my $valid_time = $self->_check_time(map $args{$_}, qw(hour minute second));
-
-        if (not $valid_date && $valid_time) {
-            my $type = !$valid_date ? 'date' : 'time';
-            $self->_set_failure;
-            $self->_set_error("(invalid $type)");
-            return $self->_get_datetime_object;
-        }
-
-        $self->_set(%args);
-        $self->_set_valid_exp;
-    }
     elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
         my ($prefix, $value, $unit) = ($1, $2, lc $3);
 
@@ -294,8 +199,6 @@ sub parse_datetime
 
         $self->_set(%args);
 
-        $self->{datetime}->truncate(to => 'second');
-        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     else {
@@ -331,7 +234,12 @@ sub _params_init
         (${$params->{string}}) = @_;
     }
 
-    trim($params->{string});
+    ${$params->{string}} = do {
+        local $_ = ${$params->{string}};
+        s/^\s+//;
+        s/\s+$//;
+        $_
+    };
 }
 
 sub _parse_init
@@ -348,7 +256,7 @@ sub _parse_init
             $self->{datetime} = dclone($self->{Datetime});
         }
         else {
-            $self->{datetime} = DateTime::HiRes->$method(
+            $self->{datetime} = DateTime->$method(
                 time_zone => $self->{Time_zone},
                 %$args,
             );
@@ -368,7 +276,6 @@ sub _parse_init
     $self->_unset_error;
     $self->_unset_valid_exp;
     $self->_unset_trace;
-    $self->_unset_truncated;
 }
 
 sub parse_datetime_duration
@@ -379,7 +286,7 @@ sub parse_datetime_duration
     $self->_params_init(@_, { string => \$duration_string });
     my $timespan_sep = $self->{data}->__timespan('literal');
 
-    my @date_strings = $duration_string =~ /\s+ $timespan_sep \s+/ix
+    my @date_strings = $duration_string =~ /\b $timespan_sep \b/ix
       ? do { $self->{duration} = true;
              split /\s+ $timespan_sep \s+/ix, $duration_string }
       : do { $self->{duration} = false;
@@ -394,12 +301,10 @@ sub parse_datetime_duration
         $shrinked = true;
     }
 
-    $self->_rewrite_duration(\@date_strings);
-
     $self->_pre_duration(\@date_strings);
-    @$self{qw(state truncated_duration)} = ({}, []);
+    $self->{state} = {};
 
-    my (@queue, @traces, @truncated);
+    my (@queue, @traces);
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
         $self->_save_state(
@@ -410,19 +315,15 @@ sub parse_datetime_duration
         if (@{$self->{traces}}) {
             push @traces, $self->{traces}[0];
         }
-        if ($self->{running_tests}) {
-            push @truncated, $self->_get_truncated;
-        }
     }
 
-    $self->_post_duration(\@queue, \@traces, \@truncated);
+    $self->_post_duration(\@queue, \@traces);
     $self->_restore_state;
 
     delete @$self{qw(duration insert state)};
 
-    @{$self->{traces}}             = @traces;
-    @{$self->{truncated_duration}} = @truncated;
-    $self->{input_string}          = $duration_string;
+    @{$self->{traces}} = @traces;
+    $self->{input_string} = $duration_string;
 
     if ($shrinked) {
         $self->_set_failure;
@@ -439,18 +340,8 @@ sub extract_datetime
     my $extract_string;
     $self->_params_init(@_, { string => \$extract_string });
 
-    $self->_unset_failure;
-    $self->_unset_error;
-    $self->_unset_valid_exp;
-
-    $self->{input_string} = $extract_string;
-
-    $self->{mode} = 'extract';
-
     my @expressions = $self->_extract_expressions($extract_string);
 
-    $self->_set_valid_exp if @expressions;
-
     return wantarray ? @expressions : $expressions[0];
 }
 
@@ -467,22 +358,8 @@ sub error
 
     return '' if $self->success;
 
-    my $error = sub
-    {
-        return undef unless defined $self->{mode} && length $self->{mode};
-        my %errors = (
-            extract => "'$self->{input_string}' cannot be extracted from",
-            parse   => "'$self->{input_string}' does not parse",
-        );
-        return $errors{$self->{mode}};
-    }->();
-
-    if (defined $error) {
-        $error .= ' ' . ($self->_get_error || '(perhaps you have some garbage?)');
-    }
-    else {
-        $error = 'neither extracting nor parsing method invoked';
-    }
+    my $error  = "'$self->{input_string}' does not parse ";
+       $error .= $self->_get_error || '(perhaps you have some garbage?)';
 
     return $error;
 }
@@ -491,7 +368,7 @@ sub trace
 {
     my $self = shift;
 
-    return @{$self->{traces} || []};
+    return @{$self->{traces}};
 }
 
 sub _process
@@ -609,7 +486,6 @@ sub _truncate
         my $index = $indexes{$unit} - 1;
         if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
             $self->{datetime}->truncate(to => $unit);
-            $self->_set_truncated;
             last;
         }
     }
@@ -622,8 +498,8 @@ sub _post_process
 
     delete $opts{truncate_to};
 
-    if (($self->{Prefer_future} || $self->{Demand_future})
-        && (exists $opts{advance_future} && $opts{advance_future})
+    if ($self->{Prefer_future} &&
+        (exists $opts{prefer_future} && $opts{prefer_future})
     ) {
         $self->_advance_future;
     }
@@ -647,41 +523,16 @@ sub _advance_future
         } @{$self->{data}->{$identifier}};
     };
 
-    my $now = exists $self->{Datetime}
-      ? dclone($self->{Datetime})
-      : DateTime::HiRes->now(time_zone => $self->{Time_zone});
-
-    my $day_of_week = sub { $_[0]->_Day_of_Week(map $_[0]->{datetime}->$_, qw(year month day)) };
-
-    my $skip_weekdays = false;
-
-    if ((all { /^(?:(?:nano)?second|minute|hour)$/ } keys %modified)
+    if ((all { /^(?:second|minute|hour)$/ } keys %modified)
         && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
-        && (($self->{Prefer_future} && $self->{datetime} <  $now)
-         || ($self->{Demand_future} && $self->{datetime} <= $now))
+        && $self->{datetime}->hour < DateTime->now(time_zone => $self->{Time_zone})->hour
     ) {
         $self->{postprocess}{day} = 1;
     }
-    elsif (sub {
-        return false unless @{$self->{tokens}} == 2;
-        my ($day, $weekday) = map $self->{data}->__RE($_), qw(day weekday);
-        if ($self->{tokens}->[0] =~ $day
-         && $self->{tokens}->[1] =~ $weekday) {
-            $skip_weekdays = true;
-            return true;
-        }
-        return false;
-    }->()
-        && (all { /^(?:day|month|year)$/ } keys %modified)
-        && (($self->{Prefer_future} && $self->{datetime}->day <  $now->day)
-         || ($self->{Demand_future} && $self->{datetime}->day <= $now->day))
-    ) {
-        $self->{postprocess}{week} = 4;
-    }
-    elsif (($token_contains->('weekdays_all') && !$skip_weekdays)
+    elsif ($token_contains->('weekdays_all')
         && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
-        && (($self->{Prefer_future} && $day_of_week->($self) <  $now->wday)
-         || ($self->{Demand_future} && $day_of_week->($self) <= $now->wday))
+        && ($self->_Day_of_Week(map $self->{datetime}->$_, qw(year month day))
+         < DateTime->now(time_zone => $self->{Time_zone})->wday)
     ) {
         $self->{postprocess}{day} = 7;
     }
@@ -692,8 +543,7 @@ sub _advance_future
               ? $self->{modified}{day} == 1
                 ? true : false
               : true)
-        && (($self->{Prefer_future} && $self->{datetime}->day_of_year <  $now->day_of_year)
-         || ($self->{Demand_future} && $self->{datetime}->day_of_year <= $now->day_of_year))
+        && ($self->{datetime}->day_of_year < DateTime->now->day_of_year)
     ) {
         $self->{postprocess}{year} = 1;
     }
@@ -727,23 +577,18 @@ sub _get_valid_exp   { $_[0]->{valid_expression}         }
 sub _set_valid_exp   { $_[0]->{valid_expression} = true  }
 sub _unset_valid_exp { $_[0]->{valid_expression} = false }
 
-sub _get_truncated   { $_[0]->{truncated}         }
-sub _set_truncated   { $_[0]->{truncated} = true  }
-sub _unset_truncated { $_[0]->{truncated} = false }
-
 sub _get_datetime_object
 {
     my $self = shift;
 
     my $dt = DateTime->new(
-        time_zone  => $self->{datetime}->time_zone,
-        year       => $self->{datetime}->year,
-        month      => $self->{datetime}->month,
-        day        => $self->{datetime}->day_of_month,
-        hour       => $self->{datetime}->hour,
-        minute     => $self->{datetime}->minute,
-        second     => $self->{datetime}->second,
-        nanosecond => $self->{datetime}->nanosecond,
+        time_zone => $self->{datetime}->time_zone,
+        year      => $self->{datetime}->year,
+        month     => $self->{datetime}->month,
+        day       => $self->{datetime}->day_of_month,
+        hour      => $self->{datetime}->hour,
+        minute    => $self->{datetime}->minute,
+        second    => $self->{datetime}->second,
     );
 
     foreach my $unit (keys %{$self->{postprocess}}) {
@@ -769,11 +614,9 @@ sub _set_datetime
 1;
 __END__
 
-=encoding ISO8859-1
-
 =head1 NAME
 
-DateTime::Format::Natural - Parse informal natural language date/time strings
+DateTime::Format::Natural - Create machine readable date/time with natural parsing logic
 
 =head1 SYNOPSIS
 
@@ -781,43 +624,30 @@ DateTime::Format::Natural - Parse informal natural language date/time strings
 
  $parser = DateTime::Format::Natural->new;
 
- $dt = $parser->parse_datetime($date_string);
- @dt = $parser->parse_datetime_duration($date_string);
-
  $date_string  = $parser->extract_datetime($extract_string);
  @date_strings = $parser->extract_datetime($extract_string);
 
+ $dt = $parser->parse_datetime($date_string);
+ @dt = $parser->parse_datetime_duration($date_string);
+
  if ($parser->success) {
      # operate on $dt/@dt, for example:
-     print $dt->strftime('%d.%m.%Y %H:%M:%S'), "\n";
+     printf("%02d.%02d.%4d %02d:%02d:%02d\n", $dt->day,
+                                              $dt->month,
+                                              $dt->year,
+                                              $dt->hour,
+                                              $dt->min,
+                                              $dt->sec);
  } else {
      warn $parser->error;
  }
 
  @traces = $parser->trace;
 
- # examples
-
- 12:14 PM
- next tuesday at 2am
- tomorrow morning
- 4pm yesterday
- 10 weeks ago
-
- 1st tuesday last november
- 2nd friday in august
- final thursday in april
-
- for 3 hours
- monday to friday
- 1 April 10 am to 1 May 8am
-
- jan 24, 2011 12:00
-
 =head1 DESCRIPTION
 
-C<DateTime::Format::Natural> parses informal natural language date/time strings.
-In addition, parsable date/time substrings may be extracted from ordinary strings.
+C<DateTime::Format::Natural> takes a string with a human readable date/time and creates a
+machine readable one by applying natural parsing logic.
 
 =head1 CONSTRUCTOR
 
@@ -830,8 +660,7 @@ not necessarily required.
            datetime      => DateTime->new(...),
            lang          => 'en',
            format        => 'mm/dd/yy',
-           prefer_future => [0|1],
-           demand_future => [0|1],
+           prefer_future => '[0|1]',
            time_zone     => 'floating',
            daytime       => { morning   => 06,
                               afternoon => 13,
@@ -852,36 +681,12 @@ Defaults to 'C<en>'.
 
 =item * C<format>
 
-Specifies the format of numeric dates.
-
-The format is used to influence how numeric dates are parsed. Given two
-numbers separated by a slash, the month/day order expected comes from
-this option. If there is a third number, this option describes where
-to expect the year. When this format can't be used to interpret the
-date, some unambiguous dates may be parsed, but there is no form
-guarantee.
-
-Current supported "month/day" formats: C<dd/mm>, C<mm/dd>.
-
-Current supported "year/month/day" formats (with slashes): C<dd/mm/yy>,
-C<dd/mm/yyyy>, C<mm/dd/yyyy>, C<yyyy/mm/dd>.
-
-Note that all of the above formats with three units do also parse
-with dots or dashes as format separators.
-
-Furthermore, formats can be abbreviated as long as they remain
-unambiguous.
-
-Defaults to 'C<d/m/y>'.
+Specifies the format of numeric dates, defaults to 'C<d/m/y>'.
 
 =item * C<prefer_future>
 
-Prefers future time and dates. Accepts a boolean, defaults to false.
-
-=item * C<demand_future>
-
-Demands future time and dates. Similar to C<prefer_future>, but stronger.
-Accepts a boolean, defaults to false.
+Turns ambiguous weekdays/months to their future relatives. Accepts a boolean,
+defaults to false.
 
 =item * C<time_zone>
 
@@ -890,7 +695,7 @@ recognized by L<DateTime>. Defaults to 'floating'.
 
 =item * C<daytime>
 
-A hash reference consisting of customized daytime hours,
+An anonymous hash reference consisting of customized daytime hours,
 which may be selectively changed.
 
 =back
@@ -899,7 +704,7 @@ which may be selectively changed.
 
 =head2 parse_datetime
 
-Returns a L<DateTime> object constructed from a natural language date/time string.
+Returns a L<DateTime> object constructed from a human readable date/time string.
 
  $dt = $parser->parse_datetime($date_string);
  $dt = $parser->parse_datetime(string => $date_string);
@@ -914,7 +719,7 @@ The date string.
 
 =head2 parse_datetime_duration
 
-Returns one or two L<DateTime> objects constructed from a natural language
+Returns one or two L<DateTime> objects constructed from a human readable
 date/time string which may contain timespans/durations. I<Same> interface
 and options as C<parse_datetime()>, but should be explicitly called in
 list context.
@@ -957,13 +762,13 @@ one string is commonly returned for durations. Useful as a debugging aid.
 The grammar handling has been rewritten to be easily extendable and hence
 everybody is encouraged to propose sensible new additions and/or changes.
 
-See the class L<DateTime::Format::Natural::Lang::EN> if you're intending
-to hack a bit on the grammar guts.
+See the classes C<DateTime::Format::Natural::Lang::[language_code]> if
+you're intending to hack a bit on the grammar guts.
 
 =head1 EXAMPLES
 
-See the class L<DateTime::Format::Natural::Lang::EN> for an overview of
-currently valid input.
+See the classes C<DateTime::Format::Natural::Lang::[language_code]> for an
+overview of currently valid input.
 
 =head1 BUGS & CAVEATS
 
@@ -1018,18 +823,6 @@ valuable suggestions and patches:
  Chifung Fan
  Xiao Yafeng
  Roman Filippov
- David Steinbrunner
- Debian Perl Group
- Tim Bunce
- Ricardo Signes
- Felix Ostmann
- Jrn Clausen
- Jim Avera
- Olaf Alders
- Karen Etheridge
- Graham Ollis
- isla w
- gibus
 
 =head1 SEE ALSO
 
